#
# Copyright 2018 Fairphone B.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Repository for the system loads.

All loads inherit from the base class :py:class:`SystemLoad` that defines the
common interface, and facilitates common behaviour.

A system load ultimately defines the implementing command to execute on a
device. Some loads need to deploy so-called assets that they require to run.
There is currently one type of asset: :py:class:`android_tools.App` that wraps
an Android app.

A system load should typically be used in the following fashion:

1. Create the system load object (instantiate it)::

        load = Monkey()

2. Iterate through its assets and make sure they are deployed on the target::

        for asset in load.assets:
            if instanceof(asset, App):
                # Install the app on the target

3. Build the command to execute on the target::

        cmd = load.build_command()

4. *Do your magic* and execute the command on the target device.
"""

from typing import List


class SystemLoad:
    """The mother of all system loads.

    A load is executed on a device, all commands and configuration are
    relative to the device itself.

    The process calling the loads might or might not use dedicated directories
    for each runs. No assumption is done regarding what the current directory is
    and where any output should be written to.

    Arguments:
        command: The base command to execute.
        arguments (optional): The list of arguments to append to the
            command.
        assets (optional): The list of assets required.

    Attributes:
        assets: The ordered list of all the :py:class:`android_tools.App`
            required.
        cmd: The base command to execute.
        cmd_args: The list of arguments to append to the command.
        log_file: The command log filename.

    """

    def __init__(self, command, arguments: List = None, assets: List = None):
        """Initialise the load."""
        self.cmd = command
        self.cmd_args = " ".join(arguments) if arguments else ""
        self.log_file = f"{command}.log"
        self.assets = assets or []

    def build_command(self) -> str:
        """Build the command to execute the load.

        Returns:
            A string describing the command to execute on a device.

        """
        return f"{self.cmd} {self.cmd_args} > {self.log_file}"


class Monkey(SystemLoad):
    """The Application Exerciser Monkey generates streams of user events.

    Arguments:
        events: The number of events to generate.
        seed (optional): The seed value for pseudo-random number generator.
        throttle (optional): The delay between events, in milliseconds. No
            delay means generating events as fast as possible.
        allowed_packages (optional): The list of packages to place on the
            whitelist. By default, all packages on the device are allowed.
    """

    def __init__(
        self,
        events: int,
        seed: int = None,
        throttle: int = None,
        allowed_packages: List[str] = None,
    ):
        """Initialise the load and configure the Monkey."""
        arguments = []
        if seed:
            arguments.append(f"-s {seed}")
        if throttle:
            arguments.append(f"--throttle {throttle}")
        if allowed_packages:
            for package in allowed_packages:
                arguments.append(f"-p {package}")
        arguments.append(str(events))
        super().__init__("monkey", arguments)
